let $container = $('.editor-container');
let $tabs = $('.editor-tabs');
let CLASS_NAMES = {
  drd: 'dmn-icon-lasso-tool',
  decisionTable: 'dmn-icon-decision-table',
  literalExpression: 'dmn-icon-literal-expression'
};

$tabs.delegate('.tab', 'click', function (e) {
  var viewIdx = parseInt(this.getAttribute('data-id'), 10);
  var view = dmnModeler.getViews()[viewIdx];
  dmnModeler.open(view);
});

// calc height
let dmnModelerHeight = $(window).height() / 2.5;
/**
 *
 * @type {DmnJS}
 */
let dmnModeler = new DmnJS({
  container: $container,
  height: dmnModelerHeight,
  keyboard: {
    bindTo: window
  }
});

dmnModeler.on('views.changed', function (event) {
  // save dmn in Session
  saveSessionDmn();
  var {views, activeView} = event;
  // clear tabs
  $tabs.empty();
  // add fullscreen
  $tabs.append($(`
      <div class="toggle-fullscreen clickable" onclick="toggleFullscreen()">
      </div>
    `));
  // add tabs
  views.forEach(function (v, idx) {
    const className = CLASS_NAMES[v.type];
    var tab = $(`
            <div class="tab clickable ${v === activeView ? 'active' : ''}" 
            data-id="${idx}" id="tab-dec-${v.element.id}">
              <span class="${className}"></span>
              ${v.element.name || v.element.id}
            </div>
          `);
    $tabs.append(tab);
  });
  // render new highlighting elements
  performHighlightFunction();
});

/**
 * Save diagram contents and print them to the console.
 */
function exportDiagram() {
  let retXml = undefined;
  dmnModeler.saveXML({format: false}, function (err, xml) {
    if (err) {
      console.log('could not export DMN 1.1 diagram');
    } else {
      console.log('exporting DMN 1.1 diagram..');
      retXml = xml;
    }
  });
  return retXml;
}

/**
 * Open diagram in our modeler instance.
 *
 * @param {String} dmnXML diagram to display
 */
function openDiagram(dmnXML) {
  // clean
  cleanDmnVerifierRoot();
  // import diagram
  dmnModeler.importXML(dmnXML, function (err) {
    if (err) {
      return console.error('could not open DMN');
    }
    // fetch currently active view
    var activeView = dmnModeler.getActiveView();
    // apply initial logic in DRD view
    if (activeView.type === 'drd') {
      var activeEditor = dmnModeler.getActiveViewer();
      // access active editor components
      var canvas = activeEditor.get('canvas');
      // zoom to fit full viewport
      canvas.zoom('fit-viewport');
    }
  });
}

function toggleFullscreen() {
  $(".dmn-js-parent").toggleClass('fullscreen');
  $("#root-dmn-verifier").toggleClass('hidden');
  $("#dmn-top").toggleClass('hidden');
  $(".editor-tabs").toggleClass('fullscreen-tabs');
  // calc height
  if ($(".dmn-js-parent").hasClass('fullscreen')) {
    $(".dmn-js-parent").css('height', '');
  } else {
    let dmnModelerHeight = $(window).height() / 2.5;
    $(".dmn-js-parent").css('height', dmnModelerHeight + 'px');
  }
  // zoom to fit full viewport
  var activeEditor = dmnModeler.getActiveViewer();
  // access active editor components
  var canvas = activeEditor.get('canvas');
  //canvas.zoom('fit-viewport');
}

// -----
let renderHighlightFunction = [];

function performHighlightFunction() {
  for (let i = 0; i < renderHighlightFunction.length; i++) {
    renderHighlightFunction[i].highlight();
  }
}

function cleanHighlightFunction() {
  $('.verification-container').removeClass('highlight-soft');
  for (let i = 0; i < renderHighlightFunction.length; i++) {
    renderHighlightFunction[i].clean();
  }
  renderHighlightFunction = [];
}

function highlightMultipleDecisions(elements) {
  elements.forEach(function (el) {
    highlightSingleDecision(el.identifier['decisionId']);
  });
}

function highlightSingleDecision(decisionId) {
  // node
  let $decisionNode = $(
      '[data-element-id=' + decisionId + '] > g > rect');
  $decisionNode.css('fill', 'darkred');
  // tab
  $('#tab-dec-' + decisionId).addClass('highlight');
}

function cleanMultipleDecisions(elements) {
  elements.forEach(function (el) {
    cleanSingleDecision(el.identifier['decisionId']);
  });
}

function cleanSingleDecision(decisionId) {
  // node
  let $decisionNode = $(
      '[data-element-id=' + decisionId + '] > g > rect');
  $decisionNode.css('fill', 'white');
  // tab
  $('#tab-dec-' + decisionId).removeClass('highlight');
}

/**
 *
 * @param {Array<VerificationEntryElement>}elements
 */
function highlightRules(elements) {
  renderHighlightFunction.push({
    highlight: function () {
      highlightMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-row-id=' + el.identifier['ruleId'] + ']').addClass(
            'highlight');
      });
    },
    clean: function () {
      cleanMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-row-id=' + el.identifier['ruleId'] + ']').removeClass(
            'highlight');
      });
    }
  });
}

/**
 *
 * @param {Array<VerificationEntryElement>}elements
 * @param {string} identifier
 */
function highlightColumns(elements, identifier) {
  renderHighlightFunction.push({
    highlight: function () {
      highlightMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-col-id=' + el.identifier[identifier] + ']').addClass(
            'highlight');
      });
    },
    clean: function () {
      cleanMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-col-id=' + el.identifier[identifier] + ']').removeClass(
            'highlight');
      });
    }
  });
}

/**
 *
 * @param {Array<VerificationEntryElement>}elements
 * @param {string} identifier
 */
function highlightDataEntries(elements, identifier) {
  renderHighlightFunction.push({
    highlight: function () {
      highlightMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-element-id=' + el.identifier[identifier] + ']').addClass(
            'highlight');
      });
    },
    clean: function () {
      cleanMultipleDecisions(elements);
      elements.forEach(function (el) {
        $('[data-element-id=' + el.identifier[identifier] + ']').removeClass(
            'highlight');
      });
    }
  });
}

/**
 *
 * @param {Array<VerificationEntryElement>}elements
 */
function highlightInputData(elements) {
  renderHighlightFunction.push({
    highlight: function () {
      elements.forEach(function (el) {
        $('[data-element-id=' + el.identifier['inputDataId'] + '] > g > rect')
        .css('fill', 'darkred');
      });
    },
    clean: function () {
      elements.forEach(function (el) {
        $('[data-element-id=' + el.identifier['inputDataId'] + '] > g > rect')
        .css('fill', 'white');
      });
    }
  });
}

/**
 *
 * @param {Array<VerificationEntryElement>}elements
 */
function highlightDecision(elements) {
  console.log(elements);
  renderHighlightFunction.push({
    highlight: function () {
      highlightMultipleDecisions(elements)
    },
    clean: function () {
      cleanMultipleDecisions(elements);
    }
  });
}
